// ARDUPILOT 2.7 Beta

#include <avr/io.h>
#include <avr/eeprom.h>
#include <math.h>
#include "defines.h"

//To use the header file in your library, use brackets:
//#include <myHeader.h>

//To use the header file in your local folder, use quotes:
//#include "AP_Config_xplane.h"
#include "AP_Config.h"
#include "AP_PID_settings.h"

 /*
ArduPilot By Jordi Munoz
ArduPilot Version 2.5 By Jason Short
ArduPilot Version 2.6 By Doug Weibel
ArduPilot Version 2.6.1, 2.6.2 By Doug Weibel & Jason Short
ArduPilot Version 2.7 By Jason Short
for FlightGear by Matti Rytkonen
Developed by
	-Chris Anderson
	-Jordi Munoz
	-Jason Short
	-Doug Weibel
	-Ryan Beall
	-HappyKillMore
	-Jose Julio	
	-Bill Premerlani
	-James Cohen.
	-JB from rotorFX.
	-Automatik.
	-Fefenin
	-Peter Meister
	-Remzibi
	-Matti Rytkonen.

Beta Testers:
	-Christof Schmid
	-PM me to get your name here

*/


// GENERAL VARIABLE DECLARATIONS
// --------------------------------------------
byte 	control_mode		= MANUAL;
boolean failsafe			= false;	// did our throttle dip below the failsafe value?
byte 	config_tool_options	= 0;		// a bitmask defining config tool options such as altitude hold

// Radio values
// ------------
int radio_in[3]		= {1500,1500,CH3_TRIM};			// current values from the transmitter - microseconds
int radio_max[] 	= {CH1_MAX, CH2_MAX, CH3_MAX};	// may be reset by init sequence
int radio_trim[] 	= {0,0,CH3_TRIM};				// may be reset by init sequence
int radio_min[] 	= {CH1_MIN, CH2_MIN, CH3_MIN};	// may be reset by init sequence
byte ch3_timer_min	= 0;
int radio_out[3]	= {1500,1500,1100};					// PWM to ROLL PITCH Servos
float servo_out[] 	= {0,0,0};						// current values to the servos - -45 to 45 degrees, except [3] is 0 to 100

int ch3_timer_trim 	= 0; 			// set by init sequence
int elevon1_trim 	= 1500;
int elevon2_trim 	= 1500;
int ch1_temp 		= 1500;			// Used for elevon mixing
int ch2_temp 		= 1500;
// for elevons radio_in[CH_ROLL] and radio_in[CH_PITCH] are equivalent aileron and elevator, not left and right elevon


//	PID Control variables
//-----------------------
float kp[]={SERVO_ROLL_P, SERVO_PITCH_P, NAV_ROLL_P, NAV_PITCH_P, THROTTLE_P, THROTTLE_FBW_P}		
	 ,ki[]={SERVO_ROLL_I, SERVO_PITCH_I, NAV_ROLL_I, NAV_PITCH_I, THROTTLE_I, THROTTLE_FBW_I}
	 ,kd[]={SERVO_ROLL_D, SERVO_PITCH_D, NAV_ROLL_D, NAV_PITCH_D, THROTTLE_D, THROTTLE_FBW_D}; 		//PID gains

const float integrator_max[]={SERVO_ROLL_INT_MAX, SERVO_PITCH_INT_MAX, NAV_ROLL_INT_MAX, NAV_PITCH_INT_MAX, THROTTLE_INTEGRATOR_MAX, THROTTLE_FBW_INT_MAX};

float	integrator[]={0,0,0,0,0,0};			// PID Integrators
float	last_error[]={0,0,0,0,0,0};			// PID last error for derivative
float 	nav_gain_scaler		= 1;			// Gain scaling for headwind/tailwind


// GPS variables
// -------------
byte 	ground_start_count	= 5;			// have we achieved first lock and set Home?
byte 	GPS_fix				= BAD_GPS;		// This variable store the status of the GPS
boolean GPS_light			= false;		// status of the GPS light
boolean GPS_new_data		= false;		// fresh data from the GPS needs to be used
const float t7				= 10000000.0;	// used to scale GPS values for EEPROM storage
long 	iTOW 				= 0; 			// GPS Millisecond Time of Week
byte	NumSats				= 0;			// number of sats
boolean imu_ok 				= false;		// the IMU is sending data correctly
float 	scaleLongUp			= 0;			// used to reverse longtitude scaling
float 	scaleLongDown		= 0;			// used to reverse longtitude scaling


// used to consruct the GPS data from Bytes to ints and longs
// ----------------------------------------------------------
union long_union {
	int32_t dword;
	uint8_t	byte[4];
} longUnion;

union int_union {
	int16_t word;
	uint8_t	byte[2];
} intUnion;


// Location & Navigation 
// ---------------------
byte 	wp_radius			= 15;			// meters
int 	ground_speed 		= 0;			// m/s * 100
long 	ground_course 		= 0;			// deg * 100 dir of plane
long	nav_bearing			= 0;			// deg * 100 : 0 to 360 current desired bearing to navigate
long 	target_bearing		= 0;			// deg * 100 : 0 to 360 location of the plane to the target
long 	crosstrack_bearing	= 0;			// deg * 100 : 0 to 360 desired angle of plane to target
int 	climb_rate 			= 0;			// m/s * 100
byte	loiter_radius 		= LOITER_RADIUS; // meters


// Airspeed
// --------
float 	airspeed 			= 0; 					// m/s
int		airspeed_cruise		= AIRSPEED_CRUISE;		// m/s : target airspeed sensor value

// Location Errors
// ---------------
long 	bearing_error		= 0; 			// deg * 100 : 18000 to -18000 
long 	altitude_error		= 0;			// meters * 100 we are off in altitude
float 	airspeed_error		= 0;			// m/s : 
float	crosstrack_error	= 0;			// deg * 100 : 180 to -180  meters we are off trackline
long 	energy_error		= 0;

// Sensors 
// --------
float 	airpressure_raw		= 511;		// Airspeed Sensor - is a float to better handle filtering
int 	airpressure_offset	= 0;		// analog air pressure sensor while still
int 	airpressure			= 0;		// airspeed as a pressure value
float 	battery_voltage 	= 1024;		// Battery Voltage

long analog0				= 511;		// Thermopiles - Pitch
long analog1				= 511;		// Thermopiles - Roll
long analog2				= 511;		// Thermopiles - Z
int ir_max					= 300;		// used to scale Thermopile output to 511
int ir_max_save				= 300;		// used to scale Thermopile output to 511
long roll_sensor			= 0;		// how much we're turning in deg * 100
long pitch_sensor			= 0;		// our angle of attack in deg * 100


// Performance
// -----------
int 	max_altitude		= 0;			// meters : read by config tool for seeing how high we've gone
byte 	max_speed			= 0;			// m/s 	  : read by config tool for seeing how fast we've gone


// flight mode specific
// --------------------
boolean takeoff_complete	= 0;			// Flag for using take-off controls


// Loiter management
// -----------------
long 	old_target_bearing	= 0;			// deg * 100
int		loiter_total 		= 0;			// deg : how many times to loiter * 360
int 	loiter_delta		= 0;			// deg : how far we just turned
int		loiter_sum			= 0;			// deg : how far we have turned around a waypoint


// these are the values for navigation control functions
// ----------------------------------------------------
long 	nav_roll			= 0;			// deg * 100 : target roll angle
long 	nav_pitch			= 0;			// deg * 100 : target pitch angle
int 	throttle_cruise		= THROTTLE_CRUISE;	// 0-125 : target throttle output for average speed
long	altitude_estimate	= 0;			// for smoothing GPS output


// Waypoints
// ---------
long 	wp_distance				= 0;		// meters - distance between plane and next waypoint
long 	wp_totalDistance		= 0;		// meters - distance between old and next waypoint
byte 	wp_total				= 0;		// # of waypoints
byte 	wp_index				= 0;		// Current WP index, -1 is RTL


// 3D Location vectors
// -------------------
struct Location {
	long lat;
	long lng;
	long alt;
};

struct Location home 			= {0,0,0};	// home location
struct Location prev_WP 		= {0,0,0};	// last waypoint
struct Location current_loc		= {0,0,0};	// current location
struct Location next_WP 		= {0,0,0};	// next waypoint
long target_altitude			= 0;		// used for 
long offset_altitude			= 0;		// used for 
int hold_alt_above_home			= 0;
//int test_alt					= 0;
// IMU specific
// ------------
#if GPS_PROTOCOL == 3


long perf_mon_timer = 0;
byte imu_health = 0;
int G_Dt_max = 0.0;						//Max main loop cycle time in milliseconds
byte gyro_sat_count = 0;
byte adc_constraints = 0;
byte renorm_sqrt_count = 0;
byte renorm_blowup_count = 0;
byte gps_payload_error_count = 0;
byte gps_checksum_error_count = 0;
byte gps_pos_fix_count = 0;
byte gps_nav_fix_count = 0;
byte gps_messages_sent = 0;
byte gps_messages_received = 0;
int imu_messages_received = 0;
byte imu_payload_error_count = 0;
byte imu_checksum_error_count = 0;
#endif

//§§§
#if GPS_PROTOCOL == 6
  char *token; //Some pointers
  char *search = ",";
  char *brkb, *pEnd;
  char gps_buffer[200]; //The traditional buffer.
  char gps_GGA[80]; // added for GPS emulator
  char gps_RMC[80]; // added for GPS emulator
  //virtual home position parameters. 
  float vhome_lat = 60.25;
  float vhome_lon = 25.04;
  unsigned int vhome_alt = 100;
  int FGmode = 0;
  int ack = 0;
  float FGlat = 0;
  float FGlon = 0;
  float asp = 0;
  float phi = 0;
  float theta = 0;
  int sim_gps_mode = 0;
  int sim_loop_waypoints = 1; //zero if we do not want to loop waypoints
#endif
//§§§



// System Timers
// --------------
unsigned long fast_loopTimer		= 0;		// Time in miliseconds of main control loop
unsigned long slow_loopTimer		= 0;		// Time in miliseconds of navigation control loop
byte medium_loopCounter				= 0;		// Counters for branching from main control loop to slower loops
byte slow_loopCounter				= 0;		// 
unsigned long deltaMiliSeconds 		= 0;		// Delta Time in miliseconds
unsigned long dTnav					= 0;		// Delta Time in milliseconds for navigation computations
int mainLoop_count 					= 0;


// Basic Initialization
//---------------------
void setup() {
	init_ardupilot();
}


void loop()
{
	// We want this to execute at 50Hz if possible
	// -------------------------------------------
	if (DIYmillis()-fast_loopTimer > 19) {
		deltaMiliSeconds 	= DIYmillis() - fast_loopTimer;
		fast_loopTimer		= DIYmillis();

		// Execute the fast loop
		// ---------------------
		fast_loop();
		
		// Execute the medium loop 
		// -----------------------
		medium_loop();
	}
}

void fast_loop()
{

	// Filters radio input - adjust filters in the radio.pde file
	// ----------------------------------------------------------
	read_radio();
	
	// check for throtle failsafe condition
	// ------------------------------------
	#if THROTTLE_FAILSAFE == 1
		throttle_failsafe();
	#endif
		
	// Read Airspeed
	// -------------
	#if AIRSPEED_SENSOR == 1 && GPS_PROTOCOL < 5  // 5 - sim
             read_airspeed();
	#endif

	// read in the plane's attitude
	// ----------------------------
	#if GPS_PROTOCOL == 3
		// We are using the IMU
		// ---------------------
		decode_gps();
		
	#elif GPS_PROTOCOL == 5
		// this is the sim
		// ---------------
		read_sim_sensors();
		
	#else
		// 	We are using FMA sensors
		// ------------------------
		read_XY_sensors();
	
		// if we are using a Z sensor, read that in too
		// --------------------------------------------
		#if ENABLE_Z_SENSOR == 1
		read_z_sensor();
		#endif
	#endif

	// altitude smoothing
	// ------------------
	calc_altitude_error();
	
	// custom code/exceptions for flight modes
	// ---------------------------------------
	update_current_flight_mode();
	
	// apply desired roll and pitch to the plane
	// -----------------------------------------
	stabilize();

	// write out the servo PWM values
	// ------------------------------
	set_servos();

	#if GCS_PROTOCOL == 3
		output_Xplane();
	#endif


	#if GPS_PROTOCOL == 3
		if (mainLoop_count != 0){
			printPerfData();
		}
	#endif
}

void medium_loop()
{
	// This is the start of the medium (10 Hz) loop pieces
	// -----------------------------------------
	switch (medium_loopCounter){
	
		// This case deals with the GPS
		//-------------------------------
		case 0:
			medium_loopCounter++;
			update_GPS();
			break;

		// This case performs some navigation computations
		//------------------------------------------------
		case 1:
			medium_loopCounter++;
			// update navigation timer
			// -----------------------			
			dTnav = DIYmillis() - slow_loopTimer;
			slow_loopTimer = DIYmillis();

			// find out how much to roll the plane based on our bearing error
			// --------------------------------------------------------------
			if(control_mode >= AUTO)
				calc_nav_roll();

			break;

		// This case reserved (maybe split telemetry and logging...)
		//---------------------------------------
		case 2:
			medium_loopCounter++;
			break;

		// This case deals with sending high rate telemetry
		//-------------------------------------------------
		case 3:
			medium_loopCounter++;
			print_attitude();
			break;
			
		// This case controls the slow loop
		//---------------------------------
		case 4:
			medium_loopCounter=0;
			slow_loop();
			break;	
	}
}

void slow_loop()
{
	// This is the slow (3 1/3 Hz) loop pieces
	//----------------------------------------
	switch (slow_loopCounter){
		case 0:
			slow_loopCounter++;

			// Read 3-position switch on radio (usually Ch5)
			// ---------------------------------------------
			read_control_switch();

			// Read main battery voltage if hooked up - does not read the 5v from radio
			// ------------------------------------------------------------------------
			#if BATTERY_EVENT == 1
			read_battery();
			#endif
			
			// Blink GPS LED if we don't have a fix
			// ------------------------------------



                  	   update_GPS_light();


		
			break;
			
		case 1:
			slow_loopCounter++;
			// calculate the plane's desired bearing
			// -------------------------------------
			navigate();
			break;
			
		case 2:
			slow_loopCounter = 0;
			// Reserved
			// Probably put uplink here
			break;
	}
}


void update_GPS(void)
{

//§§§
#if GPS_PROTOCOL == 3
//do nothing
#elseif GPS_PROTOCOL == 6

//§§§
#else
decode_gps();
#endif


#if GPS_PROTOCOL == 6
  decodeFGS();   // Decode uplink data from Flight Gear
  print_data_emulator();
#endif


	if (GPS_new_data){
		print_position();

		if(ground_start_count == 1){
			// We countdown N number of good GPS fixes
			// so that the altitude is more accurate
			// -------------------------------------
			if (current_loc.lat == 0){
				Serial.println("!! bad loc");
				ground_start_count = 5;
			}else{
				init_home();
				ground_start_count = 0;
			}
			
		}else if(ground_start_count > 1){
			ground_start_count--;
		}
				
		// save performance data to the EEPROM
		// -------------------------------------
		set_max_altitude_speed();
	}
}

void update_current_flight_mode(void)
{
	switch(control_mode){
		case RTL:
		case LOITER:
		case AUTO:
			calc_throttle();
			break;
		case FLY_BY_WIRE_A:
		case FLY_BY_WIRE_B:
			// fake Navigation output using sticks
			#if CH1_MAX == 2000 && CH2_MAX == 2000 && CH1_MIN == 1000 && CH2_MIN == 1000
				// for people not using radio trim sequence
				nav_roll	= ((radio_in[CH_ROLL]	- radio_trim[CH_ROLL])	* (long)HEAD_MAX * REVERSE_ROLL) / 300;
				nav_pitch 	= ((radio_in[CH_PITCH]  - radio_trim[CH_PITCH]) * -1 * (long)PITCH_MIN * REVERSE_PITCH) / 300;
			#else
				nav_roll	= ((radio_in[CH_ROLL]	- radio_trim[CH_ROLL])	* (long)HEAD_MAX * REVERSE_ROLL) / (radio_max[CH_ROLL] - radio_trim[CH_ROLL]);
				nav_pitch 	= ((radio_in[CH_PITCH] - radio_trim[CH_PITCH]) * -1 * (long)PITCH_MIN *	REVERSE_PITCH) / (radio_max[CH_PITCH] - radio_trim[CH_PITCH]);
			#endif
	
			nav_roll = constrain(nav_roll, -HEAD_MAX, HEAD_MAX); 
			// nav_pitch is the amount to adjust pitch due to altitude
			// We use PITCH_MIN because its magnitude is normally greater than PITCH_MAX
			nav_pitch = constrain(nav_pitch, -2000, 2000);

			#if AIRSPEED_SENSOR == 1 && GPS_PROTOCOL < 5
				if(control_mode == FLY_BY_WIRE_B){
					target_altitude = current_loc.alt + (nav_pitch);
					nav_pitch = 0;
					airspeed_error = ((AIRSPEED_FBW_MAX - AIRSPEED_FBW_MIN) * servo_out[CH_THROTTLE] / 125) + AIRSPEED_FBW_MIN;
					airspeed_error = (airspeed_error - airspeed);
					calc_throttle();
				}
			#endif
			break;
		
		case STABILIZE:
			nav_roll = 0;
			nav_pitch = 0;
			break;
		
		case TAKEOFF:
			nav_roll 	= 0;
			nav_pitch 	= TAKE_OFF_PITCH * 100;
			servo_out[CH_THROTTLE] = THROTTLE_MAX;
			break;

		case LAND:
			if(wp_index >= SLOW_WAYPOINT){
				servo_out[CH_THROTTLE] = THROTTLE_SLOW;
				airspeed_cruise = AIRSPEED_SLOW;
			}
			
			if(wp_index >= THROTTLE_CUT_WAYPOINT){
				servo_out[CH_THROTTLE] = 0;
				nav_pitch = LAND_PITCH * 100;
				//nav_roll = 0;
			}
			if(wp_index == LAND_WAYPOINT){
				servo_out[CH_THROTTLE] = 0;
				nav_pitch = LAND_PITCH * 100;
				nav_roll = 0;
			}
			break;

		case CIRCLE:
			// we have no GPS installed and have lost radio contact
			// or we just want to fly around in a gentle circle w/o GPS
			// ----------------------------------------------------
			nav_roll = HEAD_MAX / 3;
			
			if (failsafe == true){
				servo_out[CH_THROTTLE] = throttle_cruise;
			}
			break;
		
		case MANUAL:
			// apply desired roll, pitch and throttle to the plane
			// -----------------------------------------
			servo_out[CH_ROLL]  = ((radio_in[CH_ROLL]  - radio_trim[CH_ROLL]) * 45  * REVERSE_ROLL) / 400;
			servo_out[CH_PITCH] = ((radio_in[CH_PITCH] - radio_trim[CH_PITCH]) * 45 * REVERSE_ROLL) / 400;
			break;
	}

	// If we have lost gps and have no ability to navigate we will circle at a gentle bank angle
	// This is another form of failsafe, different from losing radio signal.
	if(GPS_fix == FAILED_GPS && control_mode > FLY_BY_WIRE_B){		
		nav_roll = HEAD_MAX / 3;   		
	}
}

