void save_EEPROM_groundstart(void)
{	
	// Sensor settings
	eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x01, airpressure_offset);	eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x3E4, ir_max);				eeprom_busy_wait();

	// Radio settings
	eeprom_write_word((uint16_t *)	0x3D6, radio_trim[CH_ROLL]);			eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x3D8, radio_trim[CH_PITCH]);			eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x3DA, radio_trim[CH_THROTTLE]);			eeprom_busy_wait();

	eeprom_write_word((uint16_t *)	0x3E8, elevon1_trim);		eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x3EA, elevon2_trim);		eeprom_busy_wait();

	eeprom_write_word((uint16_t *)	0x3DC, radio_min[CH_ROLL]);			eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x3DE, radio_max[CH_ROLL]);			eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x3E0, radio_min[CH_PITCH]);			eeprom_busy_wait();
	eeprom_write_word((uint16_t *)	0x3E2, radio_max[CH_PITCH]);			eeprom_busy_wait();	

	eeprom_write_word((uint16_t *)	0x3D4, ch3_timer_trim);
	
}

void restore_EEPROM(void)
{
	// if this is a first time use
	check_eeprom_defaults();
	
	// Read out user options
	// ----------------------
	eeprom_busy_wait();
	config_tool_options = eeprom_read_byte((uint8_t *)	0x00);	eeprom_busy_wait();
	wp_total 			= eeprom_read_byte((uint8_t *)	0x09);	eeprom_busy_wait();	//includes home
	wp_index 			= eeprom_read_byte((uint8_t *)  0x0A); 	eeprom_busy_wait();	// or return current waypoint
	if(WP_RADIUS == -1){
		wp_radius 			= eeprom_read_byte((uint8_t *)	0x0B);	eeprom_busy_wait();
	}else{
		wp_radius = WP_RADIUS;
	}
	airpressure_offset 	= eeprom_read_word((uint16_t *)	0x01);	eeprom_busy_wait(); 
	ir_max				= eeprom_read_word((uint16_t *)	0x3E4);	eeprom_busy_wait(); 

	radio_trim[CH_ROLL] 			= eeprom_read_word((uint16_t *)	0x3D6);	eeprom_busy_wait();
	radio_trim[CH_PITCH] 			= eeprom_read_word((uint16_t *)	0x3D8);	eeprom_busy_wait();
	radio_trim[CH_THROTTLE] 			= eeprom_read_word((uint16_t *)	0x3DA);	eeprom_busy_wait();
	ch3_timer_trim 		= eeprom_read_word((uint16_t *)	0x3D4);	eeprom_busy_wait();

	elevon1_trim 			= eeprom_read_word((uint16_t *)	0x3E8);	eeprom_busy_wait();
	elevon2_trim 			= eeprom_read_word((uint16_t *)	0x3EA);	eeprom_busy_wait();

	radio_min[CH_ROLL] 	= eeprom_read_word((uint16_t *)			0x3DC);	eeprom_busy_wait();
	radio_max[CH_ROLL] 	= eeprom_read_word((uint16_t *)			0x3DE);	eeprom_busy_wait();
	radio_min[CH_PITCH] 	= eeprom_read_word((uint16_t *)			0x3E0);	eeprom_busy_wait();
	radio_max[CH_PITCH] 	= eeprom_read_word((uint16_t *)			0x3E2);	eeprom_busy_wait();

	// lets fix broken values
	// ----------------------
	wp_index 			= constrain(wp_index, 	0, 	wp_total);
	radio_trim[CH_ROLL] 			= constrain(radio_trim[CH_ROLL], 	950, 	2050);
	radio_trim[CH_PITCH] 			= constrain(radio_trim[CH_PITCH], 	950, 	2050);
	radio_trim[CH_THROTTLE] 			= constrain(radio_trim[CH_THROTTLE], 	950, 	2050);
	elevon1_trim 		= constrain(elevon1_trim, 	950, 	2050);
	elevon2_trim 		= constrain(elevon2_trim, 	950, 	2050);
	ch3_timer_trim 		= constrain(ch3_timer_trim, -15, 125);

	wp_radius = constrain(wp_radius, 	10, 	40);
	//wp_radius = 40;

	radio_min[CH_ROLL] = constrain(radio_min[CH_ROLL], 	950, 	2050);
	radio_max[CH_ROLL] = constrain(radio_max[CH_ROLL], 	950, 	2050);
	radio_min[CH_PITCH] = constrain(radio_min[CH_PITCH], 	950, 	2050);
	radio_max[CH_PITCH] = constrain(radio_max[CH_PITCH], 	950, 	2050);
	
	ir_max	 = constrain(ir_max, 40, 512);
	airpressure_offset = constrain(airpressure_offset, 0, 512);
	
	// load home latitude, long, alt
	// -----------------------------
	home = get_wp_with_index(0);
	
	// load next WP
	// ------------
	load_waypoint();
}

void check_eeprom_defaults(void)
{
	int test = eeprom_read_word((uint16_t *)	0x3D6);	eeprom_busy_wait();
	
	if (test < 100){
		eeprom_busy_wait();
		eeprom_write_byte((uint8_t *)	0x0B, 15);		eeprom_busy_wait();	// default WP radius
		eeprom_write_byte((uint8_t *)	0x3E7, 0);		eeprom_busy_wait();	// 0 = abs, 1 = relative
		eeprom_write_byte((uint8_t *)	0x3E6, 0);		eeprom_busy_wait(); // 0 = return home after 1 trip
		eeprom_write_word((uint16_t *)	0x01,  250);	eeprom_busy_wait();	// airpressure_offset
		eeprom_write_word((uint16_t *)	0x3E4, 200);	eeprom_busy_wait();	// ir_max 

		eeprom_write_word((uint16_t *)	0x3D6, 1500);	eeprom_busy_wait();	// radio_trim[CH_ROLL]
		eeprom_write_word((uint16_t *)	0x3D8, 1500);	eeprom_busy_wait();	// radio_trim[CH_PITCH]
		eeprom_write_word((uint16_t *)	0x3DA, 1024);	eeprom_busy_wait(); // radio_trim[CH_THROTTLE]
		eeprom_write_word((uint16_t *)	0x3D4, 0);		eeprom_busy_wait(); // ch3_timer_trim
		eeprom_write_word((uint16_t *)	0x3E8, 1500);	eeprom_busy_wait();	// elevon1_trim
		eeprom_write_word((uint16_t *)	0x3EA, 1500);	eeprom_busy_wait(); // elevon2_trim
		
		eeprom_write_word((uint16_t *)	0x3DC, 1100);	eeprom_busy_wait(); // radio_min[CH_ROLL]
		eeprom_write_word((uint16_t *)	0x3DE, 1900);	eeprom_busy_wait(); // radio_max[CH_ROLL]
		eeprom_write_word((uint16_t *)	0x3E0, 1100);	eeprom_busy_wait(); // radio_min[CH_PITCH]
		eeprom_write_word((uint16_t *)	0x3E2, 1900);	eeprom_busy_wait(); // radio_max[CH_PITCH]
	}
}

void set_max_altitude_speed(void)
{
	if(get_altitude_above_home() > max_altitude) {
	  max_altitude = get_altitude_above_home();
	  eeprom_busy_wait();
	  eeprom_write_word((unsigned int*)0x05, (max_altitude/100));
	}
	
	if(ground_speed > max_speed){
	  max_speed = ground_speed;
	  eeprom_busy_wait();
	  eeprom_write_word((unsigned int*)0x07,(max_speed/100));
	}
}

/* 
ailerons
EEPROM memory map


0 0x00		byte configuration - Bitmap of groundstation toggles - still needs to be defined
1 0x01 		int airpressure_offset
2 0x02 		..
3 0x03 		byte rolltrim
4 0x04 		byte pitchtrim
5 0x05 		uint max_altitude
6 0x06 		..
7 0x07 		uint max_airspeed
8 0x08 		..
9 0x09 		byte wp_total
10 0x0A		byte wp_index
11 0x0B		byte radius
12 0x0C		int home altitude
13 0x0D		..	
14 0x0E		long home latitude
15 0x0F		..
16 0x10		..
17 0x11		..
18 0x12		long home longitude
19 0x13		..
20 0x14		..
21 0x15		..	
22 0x16		Alt_hold_Home
23 0x17 	..	
24 0x18 	long waypoint 1 latitude	
25 0x19 	..	
26 0x1A 	..	
27 0x1B 	..	
28 0x1C 	long waypoint 1 longitude
29 0x1D 	..
30 0x1E 	..
31 0x1F 	..
32 0x20 	int  waypoint 1 altitude
33 0x21 	..
34 0x22 	

...
0x1F5	TOW
0x1F6	..
0x1F7	..
0x1F8	..
...
0x3D2	
0x3D3	
0x3D4	ch3_timer_trim
0x3D5	..
0x3D6	int ch1_TRIM
0x3D7	..
0x3D8	int ch2_TRIM
0x3D9	..
0x3DA	int ch3_TRIM
0x3DB	..
0x3DC	int radio_min[CH_ROLL]
0x3DD	..
0x3DE	int radio_max[CH_ROLL]
0x3DF	..
0x3E0	int radio_min[CH_PITCH]
0x3E1	..
0x3E2	int radio_max[CH_PITCH]
0x3E3	..
0x3E4	int ir_max
0x3E5	..
0x3E6	byte loop_waypoints
0x3E7	byte waypoint_mode
0x3E8   int elevon1_trim
0x3E9   ..
0x3EA   int elevon2_trim
0x3EB   ..



















*/


