#if GCS_PROTOCOL == 1

// The output buffer
byte out_buffer[32];

void print_current_waypoints()
{
	Serial.print("DIYd");  			// Message preamble
	Serial.print(25,HEX);			// Payload length
	Serial.print(0x10);				// Message ID
	out_buffer[0] = wp_index;		// waypoint Number      
	
	output_long(1, prev_WP.lat);	// bytes 1,2,3,4
	output_long(5, prev_WP.lng);	// bytes 5,6,7,8
	output_long(9, prev_WP.alt);	// bytes 9,10,11,12

	output_long(13, next_WP.lat);	// bytes 13,14,15,16
	output_long(17, next_WP.lng);	// bytes 17,18,19,20
	output_long(21, next_WP.alt);	// bytes 21,22,23,24

	// print out the buffer and checksum
	print_buffer(25);
}

void print_control_mode(void)
{
	Serial.print("DIYd");  			// Message preamble
	Serial.print(1,HEX);			// Payload length
	Serial.print(0x11);				// Message ID

	out_buffer[0] = control_mode;	// Message ID
	print_buffer(1);
}

/*
Lat(int32) 				(decimal degrees * 10**7)
Lon(int32)				(decimal degrees * 10**7)
Alt_MSL(int16) 			(meters)
ground_speed(byte) 		(M/S)
ground_course(int16) 	(degrees)
target_bearing(int16) 	(degrees)
nav_bearing(int16) 		(degrees)
Time of Week (uint32) 	(milliseconds) 
*/

void print_position(void)
{
	// output GPS related DATA
	byte payload_length = 22;
	Serial.print("DIYd");  							// Message preamble
	Serial.print(payload_length);					// Payload length
	Serial.print(0x03);								// Message ID
	
	output_long(0, current_loc.lat);				// bytes 0,1,2,3
	output_long(4, current_loc.lng);				// bytes 4,5,6,7
	output_int(8, (int)current_loc.alt);			// bytes 8,9
	out_buffer[10] = (byte)(ground_speed/100);		// bytes 10
	output_int(11, (int)(target_bearing/100));		// bytes 13,14
	output_int(13, (int)(nav_bearing/100));			// bytes 15,16
	output_int(15, (int)(nav_roll/100));			// bytes 17,18
	output_int(17, (int)(nav_pitch/100));			// bytes 19,20
	output_int(19, (int)(nav_pitch/100));			// bytes 21,22

	// print out the buffer and checksum
	// ---------------------------------
	print_buffer(payload_length);
}

void print_attitude(void)
{
	// output real-time sensor data
	byte payload_length = 15;
	Serial.print("DIYd");  							// Message preamble
	Serial.print(payload_length);					// Payload length
	Serial.print(0x03);								// Message ID
	
	output_int(0, (int)(servo_out[CH_ROLL]/100));			// bytes 0,1
	output_int(2, (int)(servo_out[CH_PITCH]/100));			// bytes 2,3
	output_int(4, servo_out[CH_THROTTLE]);					// bytes 4,5
	output_int(6, (int)(roll_sensor/100));			// bytes 6,7
	output_int(8, (int)(pitch_sensor/100));			// bytes 8,9
	output_int(10,(int)(ground_course/100));		// bytes 10,11
	output_int(12, (int)airspeed);			// bytes 12,13
	out_buffer[14] = ir_max/2;						// bytes 14
	
	// print out the buffer and checksum
	// ---------------------------------
	print_buffer(payload_length);
}

void output_long(byte n, long value)
{
	out_buffer[n]	= value & 0xff;
	out_buffer[n+1]	= (value >> 8) & 0xff;
	out_buffer[n+2]	= (value >> 16) & 0xff;
	out_buffer[n+3]	= (value >> 24) & 0xff;
}

void output_int(byte n, int value)
{
	out_buffer[n]	= value & 0xff;
	out_buffer[n+1]	= (value >> 8) & 0xff;
}

void print_buffer(byte length)
{
	byte ck_a = 0;
	byte ck_b = 0;
	for (byte i = 0;i < length; i++){
		Serial.print (out_buffer[i]);
	}
	Serial.print('\r');
	Serial.print('\n');
	
	/*
	for (byte i = 0;i < length; i++){
		ck_a += out_buffer[i];  //Calculates checksums
		ck_b += ck_a;
	}
	Serial.print(ck_a);
	Serial.print(ck_b);
	*/
}
	
#endif