
//****************************************************************
// Function that will calculate the desired direction to fly and altitude error
//****************************************************************
void navigate()
{
  
  
//§§§  

#if GPS_PROTOCOL == 6

switch (FGmode)
{ 
case 0:
ack=0;
//control_mode=MANUAL; 
break;

case 2:
ack=0;
if (control_mode==MANUAL)
{
control_mode=AUTO;
}
break;

case 3:
ack=0;
control_mode=RTL;
break;

case 4:
if(ack==0)
{
waypoint_event(EVENT_WILL_REACH_WAYPOINT);
reached_waypoint();
ack=4; 
}
break;


case 5:
if(ack==0)
{ 
waypoint_event(EVENT_WILL_REACH_WAYPOINT);    
reached_waypoint();
ack=5;
}
break;
}
#endif
//§§§  
  
  
  
  
  
  
  
  
	// do not navigate with corrupt data
	// ---------------------------------
	if (GPS_fix == FAILED_GPS){
		if(control_mode != CIRCLE){
			nav_roll = 300;  // Set up a gentle bank
			nav_pitch = 0;
		}
		return;
	}
	
	//*********************************************************************************
	//	 Here we begin actual navigation computations
	//	 A computationally simple approximation to full flow field navigation for track following and loiter
	//*********************************************************************************

	// We only perform most nav computations if we have new gps data to work with
	// --------------------------------------------------------------------------
	if(GPS_new_data){ 

		GPS_new_data = false;
						
		// target_bearing is where we should be heading 
		// --------------------------------------------
		target_bearing 	= get_bearing(&current_loc, &next_WP);
	
		// nav_bearing is how we're actually going to get there
		// ----------------------------------------------------
		nav_bearing = target_bearing;

		// waypoint distance from plane
		// ----------------------------
		wp_distance = getDistance(&current_loc, &next_WP);
		
		if (wp_distance <= 0){
			Serial.print("WP error  ");
		 	Serial.println(wp_distance,DEC);
			print_current_waypoints();
			return;
		}

	    // Are we there yet?
	    // wp_distance is in ACTUAL meters, not the *100 meters we get from the GPS
	    // ------------------------------------------------------------------------



	    switch(control_mode){
	    	case LOITER:
				float power;
				if (wp_distance <= loiter_radius){
					nav_bearing -= 9000;
					
				}else if (wp_distance < (loiter_radius*2)) {
					power = (float)((loiter_radius*2) - wp_distance) / (float)loiter_radius;
					nav_bearing -= power * 9000;
				}		
		    	break;
		    	
		    case RTL:
		    	if(wp_distance <= (loiter_radius + 10)) { // + 10 meters
					waypoint_event(EVENT_WILL_REACH_WAYPOINT);
					reached_waypoint();
	    		}else{
	    			update_crosstrack();
	    		}
	    		break;

	    	case LAND:
		    case AUTO:
				loiter_delta = (target_bearing - old_target_bearing)/100;
				// reset the old value
				old_target_bearing = target_bearing;
				// wrap values
				if (loiter_delta > 170) loiter_delta -= 360;
				if (loiter_delta < -170) loiter_delta += 360;
				loiter_sum += loiter_delta;

				if (wp_distance < wp_radius ){
					waypoint_event(EVENT_WILL_REACH_WAYPOINT);
					reached_waypoint();
				}else{
	    			update_crosstrack();
	    			if(loiter_sum > 300){
	    				Serial.println("XXX missed WP");
						waypoint_event(EVENT_WILL_REACH_WAYPOINT);
						reached_waypoint();
	    			}else if (loiter_sum < -300){
	    				Serial.println("XXX missed WP");
						waypoint_event(EVENT_WILL_REACH_WAYPOINT);
						reached_waypoint();
	    			}
				}
				break;

		    case TAKEOFF:
				if ((current_loc.alt > (home.alt + TAKE_OFF_ALT * 100)) && (ground_start_count == 0))  {

                                        takeoff_complete = true;
					reset_control_switch();
				}
	    		break;
	    }
	    
		// Wrap the values
		// ---------------
		if (nav_bearing > 36000)	nav_bearing -= 36000;
		if (nav_bearing < 0)		nav_bearing += 36000;
	}
}


void update_crosstrack(void)
{
	// Crosstrack Error
	// ----------------
	if (abs(target_bearing - crosstrack_bearing) < 4500) {   // If we are too far off or too close we don't do track following
		crosstrack_error = sin(radians((target_bearing - crosstrack_bearing)/100)) * wp_distance;   // Meters we are off track line
		nav_bearing += constrain(crosstrack_error * XTRACK_GAIN,-XTRACK_ENTRY_ANGLE,XTRACK_ENTRY_ANGLE);
	}
}

void reset_crosstrack()
{
	crosstrack_bearing 	= get_bearing(&current_loc, &next_WP);	// Used for track following
}

int get_altitude_above_home(void)
{
	// This is the altitude above the home location
	// The GPS gives us altitude at Sea Level
	// if you slope soar, you should see a negative number sometimes
	// -------------------------------------------------------------
	return current_loc.alt - home.alt;
}

long getDistance(struct Location *loc1, struct Location *loc2)
{
	if(loc1->lat == 0 || loc1->lng == 0) 
		return -1;
	if(loc2->lat == 0 || loc2->lng == 0) 
		return -1;
	float dlat 		= (float)(loc2->lat - loc1->lat);
	float dlong  	= ((float)(loc2->lng - loc1->lng)) * scaleLongDown;
	return sqrt(sq(dlat) + sq(dlong)) * .01113195;
}

long get_alt_distance(struct Location *loc1, struct Location *loc2)
{
	return abs(loc1->alt - loc2->alt);
}

long get_bearing(struct Location *loc1, struct Location *loc2)
{
	long off_x = loc2->lng - loc1->lng;
	long off_y = (loc2->lat - loc1->lat) * scaleLongUp;
	long bearing =  9000 + atan2(-off_y, off_x) * 5729.57795;
	if (bearing < 0) bearing += 36000;
	return bearing;
}
